var Model;
(function (Model) {
    var Cell = (function () {
        function Cell(board, x, y, bomb) {
            this.board = board;
            this.x = x;
            this.y = y;
            this.bomb = bomb;
            this.open = false;
        }
        Object.defineProperty(Cell.prototype, "neighbors", {
            get: function () {
                return this.board.getNeighborsBomb(this.x, this.y);
            },
            enumerable: true,
            configurable: true
        });
        Object.defineProperty(Cell.prototype, "opened", {
            get: function () {
                return this.open;
            },
            set: function (val) {
                this.open = val;
            },
            enumerable: true,
            configurable: true
        });
        return Cell;
    })();
    Model.Cell = Cell;    
    var MineSweeper = (function () {
        function MineSweeper(listener, width, height, threshold) {
            if (typeof width === "undefined") { width = 10; }
            if (typeof height === "undefined") { height = 10; }
            if (typeof threshold === "undefined") { threshold = 0.15; }
            this.listener = listener;
            this.width = width;
            this.height = height;
            this.threshold = threshold;
            this.board = [];
            this.start = new Date();
            for(var w = 0; w < this.width; w++) {
                for(var h = 0; h < this.height; h++) {
                    var cell = new Cell(this, w, h, Math.random() < this.threshold);
                    this.board.push(cell);
                }
            }
            this.listener.onCreate(this);
        }
        MineSweeper.prototype.getCell = function (x, y) {
            var cells = this.board.filter(function (c) {
                return c.x == x && c.y == y;
            });
            return cells[0];
        };
        MineSweeper.prototype.getNeighbors = function (x, y) {
            var neighbors = this.board.filter(function (c) {
                return (c.x == x && c.y == y) == false && (x - 1 <= c.x && c.x <= x + 1) && (y - 1 <= c.y && c.y <= y + 1);
            });
            return neighbors;
        };
        MineSweeper.prototype.open = function (x, y, root) {
            if (typeof root === "undefined") { root = true; }
            var _this = this;
            var cell = this.getCell(x, y);
            if(cell.opened) {
                return;
            }
            cell.opened = true;
            if(cell.bomb) {
                this.listener.onDead(cell);
            } else {
                this.listener.onOpen(cell);
                if(cell.neighbors == 0) {
                    this.getNeighbors(x, y).forEach(function (c) {
                        return _this.open(c.x, c.y, false);
                    });
                }
            }
            if(root && this.cleared) {
                this.listener.onClear(new Date().getTime() - this.start.getTime());
            }
        };
        MineSweeper.prototype.isBomb = function (x, y) {
            return this.getCell(x, y).bomb;
        };
        Object.defineProperty(MineSweeper.prototype, "cleared", {
            get: function () {
                var closedCells = this.board.filter(function (c) {
                    return !c.opened && !c.bomb;
                });
                var closedCellExists = closedCells.length != 0;
                var openedBombs = this.board.filter(function (c) {
                    return c.opened && c.bomb;
                });
                var openedBombsExists = openedBombs.length != 0;
                return !closedCellExists && !openedBombsExists;
            },
            enumerable: true,
            configurable: true
        });
        MineSweeper.prototype.getNeighborsBomb = function (x, y) {
            var bomb = 0;
            this.getNeighbors(x, y).forEach(function (c) {
                return bomb += c.bomb ? 1 : 0;
            });
            return bomb;
        };
        Object.defineProperty(MineSweeper.prototype, "cells", {
            get: function () {
                return this.board;
            },
            enumerable: true,
            configurable: true
        });
        return MineSweeper;
    })();
    Model.MineSweeper = MineSweeper;    
})(Model || (Model = {}));
//@ sourceMappingURL=model.js.map
