/// <reference path="model.ts"/>
/// <reference path="tweetbutton.d.ts"/>
/// <reference path="jquery.d.ts"/>

module View {

	import m = Model

	// MineSweeper を描画するためのクラス。
	export class Render implements m.OnMineSweeperEventListener {

		game: m.MineSweeper;
		$parent: JQuery;

		// 実行開始。
		public run($parent: JQuery): void {
			this.$parent = $parent;
			$parent.empty();
			this.game = new m.MineSweeper(this);
		}

		// 盤が生成された時のコールバック。
		onCreate(board: m.MineSweeper): void {
			for(var w = 0; w < board.width; w++) {
				for(var h = 0; h < board.height; h++) {
					var $el = this.createCellElement(w, h);
					if(h == 0) {
						$el.addClass("head");
					}
					$el.addClass("cell");
					this.$parent.append($el);
				}
			}
		}

		// マスが開かれた時のコールバック。
		onOpen(cell: m.Cell): void {
			var $el = $("#cell-" + cell.x + "-" + cell.y);
			$el.addClass("opened").addClass("neighbor" + cell.neighbors);
		}

		// 地雷を開いた時のコールバック。
		onDead(cell: m.Cell): void {
			var $el = $("#cell-" + cell.x + "-" + cell.y);
			$el.addClass("bomb");
		}

		// クリアした時のコールバック。
		onClear(duration: number): void {
			var message = "" + this.game.width + "x" + this.game.height + ",地雷率" + this.game.threshold + "のマインスイーパを" + (duration / 1000) + "秒でクリアしました！"; 
			createTweetButton(message);
			alert("クリアしました！ツイートボタンを押してみましょう。");
		}

		// 1つのセル用の Element を生成する。
		private createCellElement(x: number, y: number): JQuery {
			var $el = $("<span>");
			$el.click(e => this.game.open(x, y));
			$el.attr("id", "cell-" + x + "-" + y);
			return $el;
		}
	}
}