var Test;
(function (Test) {
    var m = Model;
    describe("マインスイーパの制御について", function () {
        it("盤の初期化", function () {
            var board = new m.MineSweeper(createTemporaryListener(), 3, 3, 0);
            expect(board.cells.length).toBe(9);
        });
        it("クリアの確認", function () {
            var board;
            board = new m.MineSweeper(createTemporaryListener(), 3, 3, 0);
            board.open(0, 0);
            expect(board.cleared).toBeTruthy();
            board = new m.MineSweeper(createTemporaryListener(), 3, 3, 0);
            modifyCell(board, 2, 2, true);
            board.open(0, 0);
            expect(board.cleared).toBeTruthy();
        });
        it("残りありの確認", function () {
            var board = new m.MineSweeper(createTemporaryListener(), 3, 3, 0);
            modifyCell(board, 1, 2, true);
            modifyCell(board, 2, 2, true);
            board.open(0, 0);
            expect(board.cleared).toBeFalsy();
        });
        it("失敗の確認", function () {
            var board = new m.MineSweeper(createTemporaryListener(), 3, 3, 1);
            board.open(0, 0);
            expect(board.cleared).toBeFalsy();
        });
        it("イベントリスナの確認", function () {
            var result = "";
            var board = new m.MineSweeper({
                onCreate: function (board) {
                    return result += "s";
                },
                onOpen: function (cell) {
                    return result += "o";
                },
                onDead: function (cell) {
                    return result += "b";
                },
                onClear: function (duraion) {
                    return result += "e";
                }
            }, 3, 3, 0);
            modifyCell(board, 2, 2, true);
            board.open(0, 0);
            expect(result).toBe("sooooooooe");
            result = "";
            board = new m.MineSweeper({
                onCreate: function (board) {
                    return result += "s";
                },
                onOpen: function (cell) {
                    return result += "o";
                },
                onDead: function (cell) {
                    return result += "b";
                },
                onClear: function (duraion) {
                    return result += "e";
                }
            }, 3, 3, 0);
            modifyCell(board, 1, 2, true);
            modifyCell(board, 2, 1, true);
            board.open(0, 0);
            expect(result).toBe("soooo");
            board.open(0, 2);
            expect(result).toBe("sooooo");
            board.open(2, 0);
            expect(result).toBe("soooooo");
            board.open(1, 2);
            expect(result).toBe("soooooob");
            result = "";
            board = new m.MineSweeper({
                onCreate: function (board) {
                    return result += "s";
                },
                onOpen: function (cell) {
                    return result += "o";
                },
                onDead: function (cell) {
                    return result += "b";
                },
                onClear: function (duraion) {
                    return result += "e";
                }
            }, 3, 3, 0);
            modifyCell(board, 1, 2, true);
            modifyCell(board, 2, 1, true);
            board.open(0, 0);
            board.open(0, 2);
            board.open(2, 0);
            board.open(2, 2);
            expect(result).toBe("soooooooe");
        });
        it("クリア時間の確認", function () {
            var duration = -1;
            var board = new m.MineSweeper({
                onCreate: function (board) {
                },
                onOpen: function (cell) {
                },
                onDead: function (cell) {
                },
                onClear: function (d) {
                    return duration = d;
                }
            }, 3, 3, 0);
            modifyCell(board, 2, 2, true);
            board.open(0, 0);
            expect(duration).toBeGreaterThan(-1);
        });
    });
    function createTemporaryListener() {
        return {
            onCreate: function (board) {
            },
            onOpen: function (cell) {
            },
            onDead: function (cell) {
            },
            onClear: function (duraion) {
            }
        };
    }
    function modifyCell(board, x, y, bomb) {
        board.getCell(x, y).bomb = bomb;
    }
})(Test || (Test = {}));
//@ sourceMappingURL=model_test.js.map
