/// <reference path="jasmine-1.2.0/jasmine-1.2.d.ts"/>

/// <reference path="../scripts/model.ts"/>

module Test {

	import m = Model;

	describe("マインスイーパの制御について", () => {
		it("盤の初期化", () => {
			var board = new m.MineSweeper(createTemporaryListener(), 3, 3, 0);
			expect(board.cells.length).toBe(9); // 3x3 = 9
		});

		it("クリアの確認", () => {
			var board: m.MineSweeper;
			board = new m.MineSweeper(createTemporaryListener(), 3, 3, 0);

			board.open(0, 0);
			expect(board.cleared).toBeTruthy(); // 爆弾率0なので全部開いてクリア

			board = new m.MineSweeper(createTemporaryListener(), 3, 3, 0);
			modifyCell(board, 2, 2, true);

			board.open(0, 0);
			expect(board.cleared).toBeTruthy(); // 一番右下の爆弾以外開くのでクリア
		});

		it("残りありの確認", () => {
			var board = new m.MineSweeper(createTemporaryListener(), 3, 3, 0);
			modifyCell(board, 1, 2, true);
			modifyCell(board, 2, 2, true);

			board.open(0, 0);
			expect(board.cleared).toBeFalsy(); // 右下空欄あり
		});

		it("失敗の確認", () => {
			var board = new m.MineSweeper(createTemporaryListener(), 3, 3, 1);

			board.open(0, 0);
			expect(board.cleared).toBeFalsy(); // 爆弾率1なのでどこ開いても死ぬ
		});

		it("イベントリスナの確認", () => {

			var result = "";
			var board = new m.MineSweeper({
				onCreate: board => result += "s",
				onOpen: cell => result += "o",
				onDead: cell => result += "b",
				onClear: duraion => result += "e"
			}, 3, 3, 0);

			modifyCell(board, 2, 2, true);
			board.open(0, 0);
			expect(result).toBe("sooooooooe"); // 開始+1ヶ所(8個)開く


			result = "";
			board = new m.MineSweeper({
				onCreate: (board) => result += "s",
				onOpen: (cell) => result += "o",
				onDead: (cell) => result += "b",
				onClear: (duraion) => result += "e"
			}, 3, 3, 0);
			modifyCell(board, 1, 2, true);
			modifyCell(board, 2, 1, true);

			board.open(0, 0);
			expect(result).toBe("soooo"); // 開始+1ヶ所(4個)開く
			board.open(0, 2);
			expect(result).toBe("sooooo"); // +1ヶ所(+1個)
			board.open(2, 0);
			expect(result).toBe("soooooo"); // +1ヶ所(+1個)
			board.open(1, 2);
			expect(result).toBe("soooooob"); // +死


			result = "";
			board = new m.MineSweeper({
				onCreate: (board) => result += "s",
				onOpen: (cell) => result += "o",
				onDead: (cell) => result += "b",
				onClear: (duraion) => result += "e"
			}, 3, 3, 0);
			modifyCell(board, 1, 2, true);
			modifyCell(board, 2, 1, true);

			board.open(0, 0);
			board.open(0, 2);
			board.open(2, 0);
			board.open(2, 2);
			expect(result).toBe("soooooooe"); // クリア
		});

		it("クリア時間の確認", () => {
			var duration = -1;
			var board = new m.MineSweeper({
				onCreate: (board) => {},
				onOpen: (cell) => {},
				onDead: (cell) => {},
				onClear: (d) => duration = d
			}, 3, 3, 0);
			modifyCell(board, 2, 2, true);

			board.open(0, 0);
			expect(duration).toBeGreaterThan(-1); // 0より大きい
		});
	});

	function createTemporaryListener(): m.OnMineSweeperEventListener {
		return {
			onCreate: (board) => {},
			onOpen: (cell) => {},
			onDead: (cell) => {},
			onClear: (duraion) => {}
		};
	}

	function modifyCell(board: m.MineSweeper, x: number, y: number, bomb: bool): void {
		board.getCell(x, y).bomb = bomb;
	}

}